/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: cal_table.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08  imported
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

/* $Id: cal_table.h,v 1.4 2002/05/08 21:24:47 jdejong Exp $ */

#ifndef _CAL_TABLE_H
#define _CAL_TABLE_H
/*----------------------------------------------------------------------- */
/* Calibration table FITS binary tables routines                          */
/* Calibration table for optical/IR interferometer                        */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* table definition for calibration table */
{
  table_def *def;      /* basic table information */
  char   *date_obs;    /* observing date */
  int    npoly;        /* number of amplitude polynomial terms */
  long   row;          /* last row (0-rel) read or written, -1=> none. */
  int    time_col;     /* time column number (0-rel)*/
  int    interv_col;   /* time interval column number */
  int    source_col;   /* source number column number */
  int    tele_col;     /* telescope column number */
  int    poly_col;     /* amplitude polynomial column number */
  int    gain_col;     /* gain column number */
  int    delay_col;    /* delay column number */
  int    rate_col;     /* rate column number */
  int    weight_col;   /* Weight column number */
  int    ref_col;      /* ref telescope column number */
} cal_table_def;

typedef struct      /* structure used for cal_table row */
{
  double time;       /* Reference time (day) */
  float  timei;      /* time interval (sec) */
  float  gain[2];    /* calibration gain as (re, im) */
  float  delay;      /* delay correction in sec */
  float  rate;       /* rate correction in s/s */
  float  weight;     /* reliability weight */
  int    source;     /* source number */
  int    tele;       /* telescope number */
  int    ref;        /* reference telescope number */
  float  *amp_poly;  /* amplitude polynomial */
  table_row *raw;    /* generic table row */
} cal_table_row;

/* public functions */

/* create an cal_table_row structure */
cal_table_row* make_cal_table_row(cal_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an cal_table_row structure                                      */
/*   Inputs:                                                              */
/*      table     pointer to cal_table table structure                    */
/*   Returns:                                                             */
/*      An cal_table_row structure pointer ; NULL indicates failure       */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_cal_table_row (cal_table_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an cal_table_row structure                                    */
/*   Inputs:                                                              */
/*      row    pointer to the cal_table_row structure.                    */
/*----------------------------------------------------------------------- */

/* cal_table functions */
cal_table_def* 
make_cal_table_def(fitsfile *fptr, char *date_obs, int npoly, int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new cal_table in an open FITS file                            */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*      npoly     number of terms in teh amplitude polynomial             */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A cal_table_def pointer; NULL indicates failure                   */
/*----------------------------------------------------------------------- */

cal_table_def* 
open_cal_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing cal_table in the active FITS file                   */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An cal_table_def pointer; NULL indicates failure                  */
/*----------------------------------------------------------------------- */

int read_cal_table(fitsfile *fptr,  cal_table_def* table, 
			    cal_table_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an cal_table                                  */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_cal_table(fitsfile *fptr, cal_table_def *table, 
			     cal_table_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an cal_table                                  */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_cal_table(fitsfile *fptr, cal_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open cal_table, associated structures are destroyed*/
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_cal_table_def (cal_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an cal_table structure                               */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end CAL_TABLE_H */
/* */
